open Printf
open Scanf
open List
open Node
open Graph

let expected_nodes = 1000

module CommonGraph (N : NODE) =
struct
   module NodeKey = struct
      type t = N.n
      let equal = (=)
      let hash = Hashtbl.hash
   end

 (*
  * A hash table is necessary to achieve constant time access
  * (an array would have been fine but would require to map nodes to integer)
  * Adjacency lists are stored in lists. As such, adding an edge is in O(highest degree)
  *)
   module KeyTbl = Hashtbl.Make(NodeKey)
   module IntTbl = Hashtbl.Make(struct type t=int let equal=(=) let hash=Hashtbl.hash end)
   exception Found_cycle of N.n list

   let new_graph = fun () -> KeyTbl.create expected_nodes
   let add_node n g = KeyTbl.add g n []

   let add_edge directed n1 n2 g =
      KeyTbl.replace g n1 (n2::(KeyTbl.find g n1));
      if not directed then KeyTbl.replace g n2 (n1::(KeyTbl.find g n2))

   let add_safe directed n1 n2 g =
      let l=KeyTbl.find g n1 in if not (mem n2 l) then add_edge directed n1 n2 g 

   let get_nodes g = rev (KeyTbl.fold (fun node _ l->node::l) g [])

   let print_graph g = let print_line node set = N.print_node node ;
         printf " is adjacent to [" ;
         iter (fun n->N.print_node n; printf ";") set ;
         printf "]\n"
      in KeyTbl.iter print_line g

  (*
   * Breadth first search implementation using a queue
   * Used for shortest path and shortest cycle from a specific vertex
   *)
   let bfs cycle directed n1 n2 g =
      let parents = KeyTbl.create (KeyTbl.length g) in KeyTbl.add parents n1 (n1,0);
      let parent_of n = fst (KeyTbl.find parents n) in
      let rec path = function n when n=n1->[n1] | n->n::(path (parent_of n)) in
      let q = Queue.create () in Queue.add (n1,0) q ;
      try while not (Queue.is_empty q) do
         let (current,ci) = Queue.pop q in
         let neighbors = KeyTbl.find g current in
         let explore node = if not (KeyTbl.mem parents node) then
         (
            KeyTbl.add parents node (current,ci+1);
            Queue.push (node,ci+1) q
         ) else if cycle then let (p,pi) = KeyTbl.find parents node in
           if (directed && node=n1) or ((not directed) && (pi=ci or pi=ci+1))
              then raise (Found_cycle (rev_append (path current) (path node)))
         in iter (explore) neighbors
      done; try if cycle then [] else rev (path n2) with Not_found->[]
      with Found_cycle c->c

  (*
   * Strongly connected components using Kosaraju's algorithm
   * Tarjan's is faster but trickier to implement using a non-persistent
   * data structure on the graph (because we want not only the vertices
   * but the edges as well in the SCC, so we need to copy the graph anyway).
   *)
   let strongly_connected g =
      let nv = KeyTbl.length g and s = Stack.create() in
      let to_visit = KeyTbl.create nv and tg = new_graph () in
      KeyTbl.iter (fun n _->add_node n tg; KeyTbl.add to_visit n true) g;
      let rec dfs1 n = KeyTbl.remove to_visit n; let neighbors = KeyTbl.find g n in
         let explore node = if (KeyTbl.mem to_visit node) then (dfs1 node; add_edge true node n tg)
            else add_edge true node n tg
         in iter (explore) neighbors ; Stack.push n s
      in KeyTbl.iter (fun n _->dfs1 n) to_visit;
      let scc = ref [] in while not (Stack.is_empty s) do
         let subg = new_graph () and root = Stack.pop s in
         let rec dfs2 n = let neighbors = KeyTbl.find tg n in KeyTbl.remove tg n;
            let explore node = if (KeyTbl.mem tg node)
               then (add_node node subg; add_edge true node n subg; dfs2 node)
               else if KeyTbl.mem subg node then add_edge true node n subg
            in iter (explore) neighbors
         in if (KeyTbl.mem tg root) then (add_node root subg; dfs2 root; scc := (subg)::(!scc))
      done; !scc

  (*
   * For a non-directed graph, one dfs is enough
   *)
   let connected g =
      let nv = KeyTbl.length g in let to_visit = KeyTbl.create nv in
      KeyTbl.iter (fun n _->KeyTbl.add to_visit n true) g;
      let rec dfs n subg = KeyTbl.remove to_visit n; let neighbors = KeyTbl.find g n in
         let explore node = if (KeyTbl.mem to_visit node) then (add_node node subg; dfs node subg);
            add_edge true n node subg;
         in iter (explore) neighbors
      in let scc = ref [] in
      let explore n _ = let sg=new_graph() in (add_node n sg; dfs n sg; scc:=sg::(!scc))
      in (KeyTbl.iter (explore) to_visit; !scc)

  (*
   * A BFS is used on each vertice to find the shortest cycle from this vertice
   * It is not actually faster to do the search on the strongly connected components
   *)
   let get_shortest_cycle directed g = let s=ref [] in
      let fsc n _ = let c=(bfs true directed n n g) in let k=(length c) in if (length !s)=0 or (k>0 && k<length !s) then s:=c
      in (KeyTbl.iter (fsc) g; !s)

   let import_graph directed f = let fd = open_in f in try (
         let rl = fun ()->input_line fd in
         if directed then sscanf (rl()) "yes" () else sscanf (rl()) "no" ();
         let (nn,nl) = sscanf (rl()) "%u %u" (fun x y->(x,y)) in
         let g = new_graph () and nodes = IntTbl.create nn in
         for i=1 to nn do let n = N.new_node() in (add_node n g; IntTbl.add nodes i n) done;
         for i=1 to nl do let (n1,n2) = sscanf (rl()) "%u %u" (fun x y->(x,y)) in
            add_safe directed (IntTbl.find nodes n1) (IntTbl.find nodes n2) g
         done; close_in fd; g
      ) with _ -> close_in fd; failwith (sprintf "<%s> is not a valid graph descriptor" f)

   let export_graph directed f g =
      let fd = open_out f in if directed then fprintf fd "yes\n" else fprintf fd "no\n";
      let n = KeyTbl.length g and u = ref 1 and out = ref "" in
      let nids = KeyTbl.create n in KeyTbl.iter (fun n _->KeyTbl.add nids n !u;incr u) g;
      let write_link s t = let i=KeyTbl.find nids s and j=KeyTbl.find nids t in
         if directed or (j>=i) then (out:=sprintf "%s%u %u\n" !out i j; incr u) in
      let write_links source neighs = iter (fun n->write_link source n) (rev neighs) in
      (u:=0; KeyTbl.iter (write_links) g; fprintf fd "%d %d\n%s" n !u !out; close_out fd;)
end

module NonDirectedGraph (N : NODE) : (GRAPH with type n = N.n) =
struct
   include CommonGraph(N)
   type n = N.n
   type g = (n list) KeyTbl.t
   let add_link = add_safe false
   let shortest_path = bfs false false
   let shortest_cycle = get_shortest_cycle false
   let get_strongly_connected = connected
   let read_from_file = import_graph false
   let write_to_file = export_graph false
end

module DirectedGraph (N : NODE) : (GRAPH with type n = N.n) =
struct
   include CommonGraph(N)
   type n = N.n
   type g = (n list) KeyTbl.t
   let add_link = add_safe true
   let shortest_path = bfs false true
   let shortest_cycle = get_shortest_cycle true
   let get_strongly_connected = strongly_connected
   let read_from_file = import_graph true
   let write_to_file = export_graph true
end
