open Arg
open Graphtools
open Printf

module IntNode = struct
   type n = int
   let nid = ref 0
   let new_node () = incr nid; !nid
   let print_node n = Printf.printf "%d" n
   let reset () = nid := 0
end

module IG = NonDirectedGraph(IntNode)
module DG = DirectedGraph(IntNode)

let ugraph = ref None
let dgraph = ref None
let filename = ref ""
let directed = ref false
let print_graph = ref false
let print_scc = ref false
let write_scc = ref false
let shortest_path = ref false
let shortest_cycle = ref false
let path_source = ref 0
let path_target = ref 0
let sep = "===========================================\n"
let rec print_path = (function []->printf "\n"; | [x]-> printf "%d\n" x; | h::t -> printf "%d -> " h; print_path t)

let basename s = try String.sub s 0 (String.rindex s '.') with Not_found -> s;;
let ext s = try String.sub s (1+String.rindex s '.') ((String.length s)-(String.rindex s '.')-1) with _ -> "graph";;

let run_actions () = (
printf "Successfully loaded %s (%sdirected)\n" !filename (if !directed then "" else "not ");

if !directed && !print_graph then (
   printf "%s Adjacency list of %s\n" sep !filename;
   match !dgraph with Some g->print_string sep; DG.print_graph g; | None->()
);

if not (!directed) && !print_graph then (
   printf "%s Adjacency list of %s\n" sep !filename;
   match !ugraph with Some g->print_string sep;IG.print_graph g; | None->()
);

if !directed && !shortest_path then (
   printf "%s Shortest path from %d to %d\n" sep (!path_source) (!path_target);
   match !dgraph with
   | Some g->print_string sep; print_path (DG.shortest_path (!path_source) (!path_target) g);
   | None->()
);

if not (!directed) && !shortest_path then (
   printf "%s Shortest path from %d to %d\n" sep (!path_source) (!path_target);
   match !ugraph with
   | Some g->print_string sep; print_path (IG.shortest_path (!path_source) (!path_target) g);
   | None->()
);

if !directed && !print_scc then (
   printf "%s Strongly connected components of %s\n%s" sep (!filename) sep;
   match !dgraph with
   | Some g-> let scc = DG.get_strongly_connected g in List.iter (fun g->DG.print_graph g;print_string sep) scc;
   | None->()
);

if not (!directed) && !print_scc then (
   printf "%s Strongly connected components of %s\n%s" sep (!filename) sep;
   match !ugraph with
   | Some g-> let scc = IG.get_strongly_connected g in List.iter (fun g->IG.print_graph g;print_string sep) scc;
   | None->()
);

if !directed && !write_scc then (
   printf "%s Writing strongly connected components of %s\n%s" sep (!filename) sep;
   match !dgraph with
   | Some g-> let scc = DG.get_strongly_connected g and i=ref 1 in
       List.iter (fun g->DG.write_to_file (sprintf "%s%d.%s" (basename !filename) !i (ext !filename)) g;incr i;) scc;
   | None->()
);

if not (!directed) && !write_scc then (
   printf "%s Writing strongly connected components of %s\n%s" sep (!filename) sep;
   match !ugraph with
   | Some g-> let scc = IG.get_strongly_connected g and i=ref 1 in
       List.iter (fun g->IG.write_to_file (sprintf "%s%d.%s" (basename !filename) !i (ext !filename)) g;incr i;) scc;
   | None->()
);

if !directed && !shortest_cycle then (
   printf "%s Shortest cycle in %s\n%s" sep (!filename) sep;
   match !dgraph with
   | Some g-> let s = DG.shortest_cycle g in (print_path s; printf "Girth is %d\n" (List.length s);)
   | None->()
);

if not (!directed) && !shortest_cycle then (
   printf "%s Shortest cycle in %s\n%s" sep (!filename) sep;
   match !ugraph with
   | Some g-> let s = IG.shortest_cycle g in (print_path s; printf "Girth is %d\n" (List.length s);)
   | None->()
);
)
let load_graph f = (filename:=f; directed := false; IntNode.reset ();
   try ugraph := Some (IG.read_from_file f); run_actions()
   with Failure(_) -> (directed:=true; dgraph := Some (DG.read_from_file f);run_actions()));;

let op = [
	("-path", Tuple([Set(shortest_path);Set_int(path_source);Set_int(path_target)]), "a b  print shortest path from a to b");
	("-al", Set(print_graph), " print the adjacency list of the graph");
	("-scc", Set(print_scc), " print adjacency lists of strongly connected components");
	("-wscc", Set(write_scc), " write strongly connected components of <f.e> to <f1.e>..<fn.e>");
	("-girth", Set(shortest_cycle), " print the shortest cycle in the graph")
] in
parse op (load_graph) "Handle graph file descriptors";

if (!ugraph=None && !dgraph=None) then (usage op "Expected graph descriptor file not found !";raise (Invalid_argument "filename"));